@@ -1,5 +1,7 @@
 from aksharamukha import GeneralMap
 import re
+
+from aksharamukha.gimeltra import Transliterator
 from . import Convert,PostOptions,PostProcess,PreProcess
 from . import ConvertFix
 import json
@@ -15,6 +17,8 @@ import warnings
 import langcodes
 from inspect import getmembers, isfunction
 
+#import sys
+#sys.stdout = None
 
 def removeA(a):
     if a.count('a') == 1:
@@ -42,7 +46,11 @@ def auto_detect(text, plugin = False):
 
     for uchar in text:
         try:
-            scripts.append(unicodedata.name(uchar).split(' ')[0].lower())
+            script_name = unicodedata.name(uchar).split(' ')[0].lower()
+            if script_name != 'old':
+                scripts.append(script_name)
+            else:
+                scripts.append(unicodedata.name(uchar).split(' ')[1].lower())
         except ValueError:
             pass
             # print('Script not found')
@@ -71,8 +79,6 @@ def auto_detect(text, plugin = False):
         else:
             script = ''
 
-    #print('the script is')
-    #print(script)
     inputScript = script[0].upper() + script[1:]
 
     laoPali = ['ຆ', 'ຉ', 'ຌ', 'ຎ', 'ຏ', 'ຐ', 'ຑ', 'ຒ', 'ຓ', 'ຘ', 'ຠ', 'ຨ', 'ຩ', 'ຬ', '຺']
@@ -121,7 +127,7 @@ def auto_detect(text, plugin = False):
 
     elif inputScript == 'Meetei':
         inputScript = 'MeeteiMayek'
-    elif inputScript == 'Old':
+    elif inputScript == 'Persian':
         inputScript = 'OldPersian'
     elif inputScript == 'Phags-pa':
         inputScript = 'PhagsPa'
@@ -141,16 +147,36 @@ def auto_detect(text, plugin = False):
         inputScript = 'RussianCyrillic'
     elif inputScript == 'Zanabazar':
         inputScript = 'ZanabazarSquare'
+    elif inputScript == 'Syriac':
+        inputScript = 'Syre'
+        eastern_dia = 'ܲ ܵ ܝܼ ܘܼ ܸ ܹ ܘܿ'.split(' ')
+        if any(char in text for char in eastern_dia):
+            inputScript = 'Syrn'
+        western_dia = 'ܰ ܺ ܶ ّ ܽ'.split(' ')
+        if any(char in text for char in western_dia):
+            inputScript = 'Syrj'
     elif inputScript == 'Arabic':
-        inputScript = 'Urdu'
+        inputScript = 'Arab'
+        persian_char = 'چ گ ژ پ هٔ'.split(' ')
+        if any(char in text for char in persian_char):
+            inputScript = 'Arab-Fa'
+        urdu_char = 'ڈ ٹ ڑ ھ'.split(' ')
+        if any(char in text for char in urdu_char):
+            inputScript = 'Urdu'
         shahmukh_char = 'ݨ لؕ مھ نھ یھ رھ لھ وھ'.split(" ")
         if any(char in text for char in shahmukh_char):
             inputScript = 'Shahmukhi'
     elif inputScript == 'Latin':
         diacritics = ['ā', 'ī', 'ū', 'ṃ', 'ḥ', 'ś', 'ṣ', 'ṇ', 'ṛ', 'ṝ', 'ḷ', 'ḹ', 'ḻ', 'ṉ', 'ṟ', 'ṭ', 'ḍ', 'ṅ', 'ñ']
         Itrans = ['R^i', 'R^I', 'L^i', 'L^I', '.N', '~N', '~n', 'Ch', 'sh', 'Sh']
+        semitic = ['ʾ', 'ʿ', 'š', 'w']
+        BurmeseLOC = ['´', '˝', 'ʻ']
         if 'ʰ' in text:
             inputScript = 'Titus'
+        elif any(char in text for char in semitic):
+            inputScript = 'Latn'
+        elif any(char in text for char in BurmeseLOC):
+            inputScript = 'IASTLOC' ## Change for other LoC schemese
         elif any(char in text for char in diacritics):
             if 'ē' in text or 'ō' in text or 'r̥' in text:
                 inputScript = 'ISO'
@@ -160,6 +186,12 @@ def auto_detect(text, plugin = False):
             inputScript = 'Itrans'
         else:
             inputScript = 'HK'
+    elif inputScript in GeneralMap.IndicScripts or inputScript in GeneralMap.LatinScripts or inputScript in ['Hiragana', 'Katakana']:
+        pass
+    else:
+        from . import gimeltra
+        tr = gimeltra.Transliterator()
+        inputScript = tr.auto_script(text)
 
     return inputScript
 
@@ -193,6 +225,56 @@ def detect_preoptions(text, inputScript):
 def convert(src, tgt, txt, nativize, preoptions, postoptions):
     tgtOld = ""
 
+    ## ALA-LC Romanizations
+    ## If associated script of LoC is not in Source, convert source into that script. The convert it to the actual target
+    ## Text (non-Burmese) -> Burmese -> LoC
+    if tgt == 'IASTLOC' and src != 'Burmese':
+        txt = convert(src, "Burmese", txt, nativize, preoptions, postoptions)
+        src = 'Burmese'
+
+    ## process as follows, if source is associated with LoC script
+    if tgt == 'IASTLOC' and src == 'Burmese':
+        # the below order for preoptions is important
+        preoptions = preoptions + [tgt + src + 'Target']
+        postoptions =  [tgt + src + 'Target'] + postoptions
+        nativize = False
+
+    ## if target is associated with the loc Script
+    if src == 'IASTLOC' and tgt == 'Burmese':
+        preoptions = [src + tgt + 'Source'] + preoptions
+        postoptions =  [src + tgt + 'Source'] + postoptions
+        nativize = False
+
+    ## else convert the Loc input into the associated script and then convert it into Loc
+    ## Text (Loc) -> Burmese -> non-Burmese
+    if src == 'IASTLOC' and tgt != 'Burmese':
+        txt = convert(src, "Burmese", txt, nativize, preoptions, postoptions)
+        src = 'Burmese'
+
+    ## Semitic ISO Standards
+    ### same as below use intermediates to convert non-associated input/output
+    if tgt in GeneralMap.semiticISO.keys():
+        if GeneralMap.semiticISO[tgt] != src:
+            txt = convert(src, GeneralMap.semiticISO[tgt], txt, nativize, preoptions, postoptions)
+            src = GeneralMap.semiticISO[tgt]
+
+        if GeneralMap.semiticISO[tgt] == src:
+            preoptions = [tgt + 'Target'] + preoptions
+            postoptions =  [tgt + 'Target'] + postoptions
+            nativize = False
+            tgt = 'Latn'
+
+    if src in GeneralMap.semiticISO.keys():
+        if GeneralMap.semiticISO[src] == tgt:
+            #print('here1')
+            preoptions = [src + 'Source'] + preoptions
+            postoptions = [src + 'Source'] + postoptions
+            src = 'Latn'
+        else:
+            #print('here2')
+            txt = convert(src, GeneralMap.semiticISO[src], txt, nativize, preoptions, postoptions)
+            src = GeneralMap.semiticISO[src]
+
     if src == "Itrans" and '##' in txt:
         textNew = [[i,word] for i, word in enumerate(txt.split("##")) if (i%2 == 0)]
         textRest = [(i,word) for i, word in enumerate(txt.split("##")) if (i%2 == 1)]
@@ -204,7 +286,72 @@ def convert(src, tgt, txt, nativize, preoptions, postoptions):
     if preoptions == [] and postoptions == [] and nativize == False and src == tgt:
         return txt
 
-    if src == tgt and (src != 'Hiragana' and src != 'Katakana'):
+    ## Semitic Switches - Add Shahmukhi/Sindhi later
+    IndicSemiticMapping = { 'Hebrew': 'Hebr', 'Thaana': 'Thaa', 'Urdu': 'Arab-Ur', 'Shahmukhi': 'Arab-Pa'}
+
+    if (tgt in GeneralMap.SemiticScripts or tgt in GeneralMap.semiticISO.keys()) and src in IndicSemiticMapping.keys():
+        src = IndicSemiticMapping[src]
+    if (src in GeneralMap.SemiticScripts or src in GeneralMap.semiticISO.keys()) and tgt in IndicSemiticMapping.keys():
+        tgt = IndicSemiticMapping[tgt]
+    if src in IndicSemiticMapping.keys() and tgt in IndicSemiticMapping.keys():
+        src = IndicSemiticMapping[src]
+        tgt = IndicSemiticMapping[tgt]
+
+    # preserve semitic pattern
+    if not nativize and src == 'Hebrew':
+        src = 'Hebr'
+    if not nativize and src == 'Urdu':
+        src = 'Arab-Ur'
+    if not nativize and src == 'Shahmukhi':
+        src = 'Arab-Pa'
+    if not nativize and src == 'Thaana':
+        src = 'Thaa'
+
+    if src in ['Arab-Ur', 'Arab-Pa'] and tgt in GeneralMap.IndicScripts:
+        txt += '\u05CD'
+
+    ## Semitic to Urdu this is not needed
+    #if src in GeneralMap.SemiticScripts and src not in ['Arab-Fa', 'Arab', 'Latn'] and (tgt in ['Urdu', 'Shahmukhi']):
+    #    tgt = 'Arab-Fa'
+
+    ## recheck this the logic is missing :: This is not required
+    """if src in ['Urdu', 'Shahmukhi'] and tgt == 'Arab':
+        #print('Semiticizing Urdu')
+        txt = PreProcess.semiticizeUrdu(txt)
+        txt = PreProcess.ArabizePersian(txt)
+        src = "Arab-Fa"
+        tgt = "Arab"
+
+    if src in ['Urdu', 'Shahmukhi'] and tgt == 'Arab-Fa':
+        txt = PreProcess.semiticizeUrdu(txt)
+        src = "none"
+        tgt = "none"
+
+    if src == 'Arab-Fa' and tgt == 'Arab':
+        txt = PreProcess.ArabizePersian(txt)
+        #src = "none"
+        #tgt = "none"
+
+    if src  == 'Arab' and tgt == 'Arab-Fa':
+        txt = PreProcess.perisanizeArab(txt)
+        src = "none"
+        tgt = "none"
+
+    if src  == 'Arab' and tgt in ['Urdu', 'Shahmukhi']:
+        txt = PreProcess.perisanizeArab(txt)
+        src = "none"
+        tgt = "Arab-Fa"
+
+    if src in 'Arab-Fa' and tgt in ['Urdu', 'Shahmukhi']:
+        src = "none"
+        tgt = "none" """
+
+    ## Semitic Naivization : remove pesky Nuktas
+    if nativize:
+        if src in GeneralMap.SemiticScripts and tgt in GeneralMap.IndicScripts:
+            txt += "\u05CC"
+
+    if src == tgt and (src != 'Hiragana' and src != 'Katakana') and src not in GeneralMap.SemiticScripts:
         tgtOld = tgt
         tgt = "Devanagari"
 
@@ -232,12 +379,24 @@ def convert(src, tgt, txt, nativize, preoptions, postoptions):
         nativize = False
     if 'SoyomboFinals' in postoptions and tgt == 'Soyombo':
         txt = '\u02BE' + txt
+    #if src not in GeneralMap.SemiticScripts and tgt == 'Arab' and not nativize:
+        #postoptions.append('arabicRemoveAdditionsPhonetic')
 
     for options in preoptions:
       txt = getattr(PreProcess, options)(txt)
 
+    ### Semitic Switches
+    if 'novowelshebrew' in preoptions and src == 'Hebr':
+        txt = txt.replace('\u05B7', '')
+
     srcOld = ""
 
+    ### retain Latin ###
+    if (src != 'Latn'and src != 'Type' and src in GeneralMap.SemiticScripts) or \
+        (src in GeneralMap.IndicScripts and tgt in GeneralMap.SemiticScripts and src not in GeneralMap.LatinScripts) or \
+            (src in ['Hiragana', 'Katakana']):
+        txt = PreProcess.retainLatin(txt)
+
     if src == 'Hiragana' or src == 'Katakana':
         txt = PreProcess.JapanesePreProcess(src, txt, preoptions)
         srcOld = 'Japanese'
@@ -260,6 +419,8 @@ def convert(src, tgt, txt, nativize, preoptions, postoptions):
     else:
         transliteration = Convert.convertScript(txt, src, tgt)
 
+    #print(transliteration)
+
     if srcOld == 'Japanese' and tgt != 'Devanagari' and 'siddhammukta' not in postoptions:
         transliteration = Convert.convertScript(transliteration, "Devanagari", "ISO")
 
@@ -267,12 +428,21 @@ def convert(src, tgt, txt, nativize, preoptions, postoptions):
         tgt = tgtOld
         transliteration = Convert.convertScript(transliteration, "Devanagari", tgt)
 
+    ## apply phonetic mapping for Arabic beforehand if not semitic source
+    if (src not in GeneralMap.SemiticScripts and tgt == 'Arab' and nativize) or 'arabicRemoveAdditionsPhonetic' in postoptions:
+        transliteration = getattr(PostProcess, 'arabicRemoveAdditionsPhonetic')(transliteration)
+
+    #print(transliteration)
+
     if nativize:
-      transliteration =  PostOptions.ApplyScriptDefaults(transliteration, src, tgt)
-      if tgt != 'Tamil':
-        transliteration = PostProcess.RemoveDiacritics(transliteration)
-      else:
-        transliteration = PostProcess.RemoveDiacriticsTamil(transliteration)
+      transliteration =  PostOptions.ApplyScriptDefaults(transliteration, src, tgt, postoptions)
+      if tgt != 'Latn':
+        if tgt != 'Tamil':
+            transliteration = PostProcess.RemoveDiacritics(transliteration)
+        else:
+            transliteration = PostProcess.RemoveDiacriticsTamil(transliteration)
+
+    #print(transliteration)
 
     if 'RemoveDiacritics' in postoptions:
       if tgt == 'Tamil':
@@ -294,6 +464,11 @@ def convert(src, tgt, txt, nativize, preoptions, postoptions):
         textConv = collections.OrderedDict(sorted(textConv.items()))
         transliteration = "".join(textConv.values())
 
+    ### return Latin ###
+    transliteration = PreProcess.retainLatin(transliteration, reverse=True)
+
+    #print(transliteration)
+
     return transliteration
 
 def process(src, tgt, txt, nativize = True, post_options = [], pre_options = [], param="default"):
@@ -315,7 +490,7 @@ def convert_default(src, tgt, txt, nativize = True, post_options = [], pre_optio
     with open(dir_path + "/yaml/aksharamukha-scripts.yaml", 'r') as stream:
         data_loaded = yaml.safe_load(stream)
 
-    scriptList = GeneralMap.IndicScripts + GeneralMap.LatinScripts
+    scriptList = GeneralMap.IndicScripts + GeneralMap.LatinScripts + GeneralMap.SemiticScripts
 
     preOptionList = list(map(lambda x: x[0], getmembers(PreProcess, isfunction)))
     preOptionListLower = list(map(lambda x: x.lower(), preOptionList))
@@ -348,6 +523,7 @@ def process_default(src, tgt, txt, nativize, post_options, pre_options):
 
     if src == 'autodetect':
         src = auto_detect(txt)
+        #print('The autodetection is ' + src)
         pre_options = detect_preoptions(txt, src)
     elif src.lower() in scriptListLower:
         src = [script_id for script_id in scriptList if src.lower() == script_id.lower()][0]
@@ -372,6 +548,13 @@ def process_script_tag(src_tag, tgt_tag, txt, nativize, post_options, pre_option
     src = []
     tgt = []
 
+    if src_tag == 'Syrc':
+        src_tag = 'Syre'
+        warnings.warn("Please specify the variety of Syriac script for the source: Estrangelo (Syre), Eastern (Syrn) or Wester (Syrj). Defaulting to Syre")
+    if tgt_tag == 'Syrc':
+        tgt_tag = 'Syre'
+        warnings.warn("Please specify the variety of Syriac script for the target: Estrangelo (Syre), Eastern (Syrn) or Wester (Syrj). Defaulting to Syre")
+
     # loop through yaml to find match
     for scrpt in data_loaded.keys():
         scrpt_tag = data_loaded[scrpt]['script']
@@ -384,6 +567,7 @@ def process_script_tag(src_tag, tgt_tag, txt, nativize, post_options, pre_option
             population = 0
             lang = ''
 
+        #print(scrpt_tag, lang_tag)
         if scrpt_tag in data_loaded_wiki.keys() and lang_tag in data_loaded_wiki[scrpt_tag].keys():
             population = data_loaded_wiki[scrpt_tag][lang_tag]['population']
         else:
@@ -396,6 +580,7 @@ def process_script_tag(src_tag, tgt_tag, txt, nativize, post_options, pre_option
         if '-' not in tgt_tag and data_loaded[scrpt]['script'].lower() == tgt_tag.lower():
             tgt.append((population, scrpt))
 
+        #print(population)
         # if hypthenated find the exact match
         if '-' in tgt_tag:
             lang_part = tgt_tag.split('-')[0].lower()
@@ -438,11 +623,15 @@ def process_script_tag(src_tag, tgt_tag, txt, nativize, post_options, pre_option
 
     if len(src) > 0:
         src_pop = sorted(src, reverse=True)[0][1]
+    elif src_tag in GeneralMap.SemiticScripts:
+            src_pop = src_tag
     else:
         raise Exception('Source script code: ' + src_tag + ' not found')
 
     if len(tgt) > 0:
         tgt_pop = sorted(tgt, reverse=True)[0][1]
+    elif tgt_tag in GeneralMap.SemiticScripts:
+        tgt_pop = tgt_tag
     else:
         raise Exception('Target script code: ' + tgt_tag + ' not found')
 
@@ -594,4 +783,12 @@ def process_lang_name(src_name, tgt_name, txt, nativize, post_options, pre_optio
 
     return process_lang_tag(src, tgt, txt, nativize, post_options, pre_options)
 
+def get_semitic_json():
+    from pathlib import Path
+    cwd = Path(Path(__file__).parent)
+    with open(Path(cwd, "json/gimeltra_data.json"), "r", encoding="utf-8") as f:
+        data = json.load(f)
+
+    return data
+
 ## add the new libraries to requiesments.txt in both folders
\ No newline at end of file
